public class Station { 
    String  name;
    Edge[]  edges;
    Line    line;
    boolean visited;
    int     minTime;
    int     x;
    int     y;
    int     distance;
    Edge    optimal;

    String getShortName() { 
        int i = name.indexOf('[');
        return i < 0 ? name : name.substring(0, i);
    }

    void addEdge(Station dst, int distance) { 
        appendEdge(new Edge(dst, distance));
    }

    void addCrossEdge(Station dst, int distance) { 
        appendEdge(new CrossEdge(this, dst, distance));
    }
         
    private final void appendEdge(Edge edge) { 
        if (edges == null) { 
            edges = new Edge[1];
        } else { 
            Edge[] newEdges = new Edge[edges.length+1];
            System.arraycopy(edges, 0, newEdges, 0, edges.length);
            edges = newEdges;
        }
        edges[edges.length-1] = edge;
    }

    void reset() { 
        minTime = Integer.MAX_VALUE;
    }

    int traverse(Station target, int period, int time, int crossPenalty) {
        if (this == target) { 
            return time;
        }
        int min = Integer.MAX_VALUE;
        if (time < minTime && !visited) { 
            int minIndex = 0;
            visited = true;
            for (int i = 0; i < edges.length; i++) { 
                int result = edges[i].dst.traverse(target, 
                                                   period, 
                                                   time + edges[i].getTime(period, crossPenalty),
                                                   crossPenalty);
                if (result < min) { 
                    min = result;
                    minIndex = i;
                }
            }
            visited = false;
            optimal = edges[minIndex];
            minTime = time;
        }
        return min;
    }

    Station(String name, int distance, int x, int y) {
        this.name = name;
        this.distance = distance;
        this.x = x;
        this.y = y;
    }
}
